




handle_genomic_data <- function(AYT){
  ## making covariance matrices
  
  # AYT genotypes
  AYT_genoA <- pullSnpGeno(pop = AYT, snpChip = 1) # Change AYT to parents
  
  AYT_genoD <- AYT_genoA
  # Set each entry to zero if homozygous and 1 if heterozygous
  AYT_genoD[AYT_genoD == 2] <- 0
  
  # Remove markers with low variability because there is no information in the low variability markers
  nMar = nSnp * nChr
  AlleleFreqA = rep(NA, times = nMar)
  AlleleFreqD = rep(NA, times = nMar)
  for (Mar in 1L:nMar) {
    AlleleFreqA[Mar] = mean(AYT_genoA[, Mar]) / 2
    AlleleFreqD[Mar] = mean(AYT_genoD[, Mar]) / 2
  }
  # hist(AlleleFreq)
  TestA = AlleleFreqA < 0.05 | AlleleFreqA > 0.95
  TestD = AlleleFreqD < 0.05 | AlleleFreqD > 0.95
  
  # sum(Test)
  AYT_genoA = AYT_genoA[, !TestA]
  AlleleFreqA = AlleleFreqA[!TestA]
  
  #AYT_genoD = AYT_genoD[, !TestD]
  #AlleleFreqD = AlleleFreqD[!TestD]
  
  
  # Create empirical genomic covariance matrix for both dominance and additive effects
  
  # Additive
  nIndA = nrow(AYT_genoA)
  nMarA = ncol(AYT_genoA)
  for (Mar in 1L:nMarA) {
    AYT_genoA[, Mar] = AYT_genoA[, Mar] - 2 * AlleleFreqA[Mar]
  }
  GenomicCovA = tcrossprod(AYT_genoA) / (2 * sum(AlleleFreqA * (1 - AlleleFreqA)))
  # GenomicPrecA = solve(GenomicCovA + diag(x = GenomicCovFudge, nrow = nIndA, ncol = nIndA))
  
  # Dominance
  nIndD = nrow(AYT_genoD)
  nMarD = ncol(AYT_genoD)
  for (Mar in 1L:nMarD) {
    AYT_genoD[, Mar] = AYT_genoD[, Mar] - 2 * AlleleFreqD[Mar]
  }
  GenomicCovD = tcrossprod(AYT_genoD) / (2 * sum(AlleleFreqD * (1 - AlleleFreqD)))
  #GenomicPrecD = solve(GenomicCovD + diag(x = GenomicCovFudge, nrow = nIndD, ncol = nIndD))
  
  # Epistasis
  GenomicCovE = GenomicCovA * GenomicCovA
  # GenomicPrecE = solve(GenomicCovE)
  
  
  return(
    list(
      V_a = GenomicCovA,
      V_d = GenomicCovD,
      V_e = GenomicCovE
    )
  )
  
  
}


