

# we provide a framework for fitting the models
# you must input the data and relevant values yourself

# load libraries
library(rstan)
library(nloptr)

# load necessary functions
source("necessary_functions.R")


stan_testdata <- make_stan_ml_data(test1, 1, 1, "stan", 1, 1, 1)


##### fit maximum likelihood
ml_data <- list(
  
  # the phenotype data (response), can contain missing values
  y = 0,
  
  # vector with the observations that should be included in the likelihood, for example for cross-validation
  # if the data has missing observations, they cannot be included in this vector
  which_in_likelihood = 0,
  
  # 1 = model A, 2 = model AD, 3 = model ADX
  model_number = 0,
  
  # initial values for the optimizer for each model parameter (including the intercept)
  startvalues = 0,
  
  # covariance matrices for the additive, dominance and epistasis effects
  V_A = NULL,
  V_D = NULL,
  V_X = NULL
  
)

result_ml <- fit_ML(ml_data)



##### fit Bayesian models

# compile the stan-model
stan_mod <- stan_model(file = "wheat_a_ad_adx.stan", auto_write = TRUE)

# choose model 
# 1 = model A, 2 = model AD, 3 = model ADX
mod_nr <- 0

# loading the prior information (choose the relevant one)
prior <- readRDS("priors_bayesian.rds")

bayesian_data <- list(
  
  # number of observations
  n = 0,
  
  # the phenotype data (response) with length n, can contain missing values
  y = 0,
  
  # vector with the observations that should be included in the likelihood, for example for cross-validation
  # if the data has missing observations, they cannot be included in this vector
  which_in_likelihood = 0,
  
  # number of observation included in the likelihood
  n_in_likelihood = 0,
  
  # 1 = model A, 2 = model AD, 3 = model ADX
  model_number = mod_nr,
  
  # 3D array with lower cholesky decomposition of the covariance matrices (dim = c(model_number, n, n))
  L = NULL,
  
  # parameters for the prior
  n_knots = prior[[mod_nr]]$n_knots,
  knots = prior[[mod_nr]]$knots,
  prior_coeffs = prior[[mod_nr]]$prior_coeffs
  
)


result_bayesian <- fit_stan(data = bayesian_data,
                            stan_mod = stan_mod, 
                            use_likelihood = 1, 
                            n_burnin = 2500, 
                            n_samp = 5000)



