
// prior 1: HD prior with even basemodel (3 orderings) NOT USED
// prior 2: HD prior with uneven basemodel (3 orderings)
// prior 3: dirichlet on triple split, PC prior on the last split NOT USED
// prior 4: dirichlet on all weights (HD default)
// prior 5: old PC pior on all variances, expert
// prior 6: old PC pior on all variances, default

functions {
  
    // calculates the joint PC prior
    real eval_spline_lpdf(real logitw, vector knots, matrix[] prior_coeffs, int idx, int n_knots){
        real dx;
        real val;
        int idd;
        int low;
        int high;
        int med;
        
        // Search for correct cubic polynomial
        low = 1;
        high = n_knots;
        while(high-low > 1){
            med = (high+low)/2;
            if(logitw < knots[med]){
                high = med;
            } else{
                low = med;
            }
        }
        idd = low;
        dx = logitw-knots[idd];
        val =  prior_coeffs[idx, idd, 1];
        val += prior_coeffs[idx, idd, 2]*dx;
        val += prior_coeffs[idx, idd, 3]*dx*dx;
        val += prior_coeffs[idx, idd, 4]*dx*dx*dx;
    
        return(val);
    }
    
    // the old pc prior on log variance
    real pc_logvar_lpdf(real logvar, real u0, real alpha){
        real lambda;
        real val;
        
        // Calculate rate parameter
        lambda = -log(alpha)/u0;
        
        // Calculate prior on standard deviation
        val = gamma_lpdf(exp(0.5*logvar) | 1, lambda);
        
        // Add jacobian
        val += -log(2) + logvar/2;
        
        return(val);
    }
    
    // inverse gamma prior on log variance
    real invgamma_logvar_lpdf(real logvar, real a, real b){
        real val;
        
        // Calculate prior on precision
        val = gamma_lpdf(exp(-logvar) | a, b);
        
        // Add jacobian
        val += -logvar;
        
        return(val);
    }
    
    // gamma distribution on log variance
    real gamma_logvar_lpdf(real x, real a){ // gamma distribution on log variance
        real b = 1;
        return a*log(b) - lgamma(a) + a*x - b*exp(x);
    }
    
    // jeffreys prior
    real jeffrey_logvar_lpdf(real logvar){ // on log variance
      return 0;
    }
    
    // choosing correct prior, not used
    real prior_lpdf(real x, int priorN){
        if(priorN == 5){ // old PC
            return(pc_logvar_lpdf(x | 1, 0.01));
        } else if(priorN == 6){ // gamma
            return(invgamma_logvar_lpdf(x | 1, 5e-5));
        }
        return 0;
    }
    
    // calculates the jacobian from V/w to log variances, and then takes the determinant, and log of the determinant
    real calc_log_jacdet_regtovar(real[] theta){
      
      matrix[3,3] jacobian;
      real V;
      
      V = exp(theta[1]) + exp(theta[2]) + exp(theta[3]);
      
      jacobian[1,1] = exp(theta[1]);
      jacobian[1,2] = exp(theta[2]);
      jacobian[1,3] = exp(theta[3]);
      
      jacobian[2,1] = -exp(theta[1]) * (exp(theta[2]) + exp(theta[3])) / (V^2);
      jacobian[2,2] = (exp(theta[1]+theta[2])) / (V^2);
      jacobian[2,3] = (exp(theta[1]+theta[3])) / (V^2);
      
      jacobian[3,1] = 0;
      jacobian[3,2] = -(exp(theta[2] + theta[3])) / ((exp(theta[2]) + exp(theta[3]))^2);
      jacobian[3,3] = (exp(theta[2] + theta[3])) / ((exp(theta[2]) + exp(theta[3]))^2);
      
      return log(determinant(jacobian));
      
    }
    
    // calculates the joint prior for theta (not including covariates)
    real new_pc_prior_lpdf(real[] theta, vector knots, matrix[] prior_coeffs, int n_knots, int use_likelihood){
      
      real logdens;
      real logV;
      real logitw1; // amount of total to genetic variance
      real logitw2; // amount of genetic to dominance variance
      
      // initializing
      logdens = 0;
      
      logV = log(sum(exp(theta)));
      // note that logit(x/(x+y)) = log(x) - log(y)
      logitw1 = log(sum(exp(theta[2:3]))) - theta[1];
      logitw2 = theta[3] - theta[2];
      
      // if we do not use the likelihood we must restrict the total variance since we use the jeffreys prior
      if (use_likelihood == 0){
        logdens += normal_lpdf(logV | 0, 1);
      } else {
        logdens += jeffrey_logvar_lpdf(logV); // V, total variance
      }
      
      // adding prior for the three logit weights
      logdens += eval_spline_lpdf(logitw1 | knots, prior_coeffs, 1, n_knots); // w1, genetic vs residual
      logdens += eval_spline_lpdf(logitw2 | knots, prior_coeffs, 2, n_knots); // w2, dominance vs additive
      
      // jacobian from internal scale to regular scale (log/logit to V/w)
      logdens += logV + log(sum(exp(theta[2:3]))) - sum(theta);
      
      // jacobian from regular scale to variances (from V/w to var)
      logdens += calc_log_jacdet_regtovar(theta);
      
      return(logdens);
      
    }
    
    // calculates the joint prior for theta (not including covariates)
    real new_pc_prior_diri_lpdf(real[] theta, vector knots, matrix[] prior_coeffs, int n_knots, int use_likelihood){
      
      real logdens;
      real logV;
      real logitw1;
      real logitw2;
      real logitw3;
      
      // initializing
      logdens = 0;
      
      logV = log(sum(exp(theta)));
      // note that logit(x/(x+y)) = log(x) - log(y)
      logitw1 = log(sum(exp(theta[2:4]))) - theta[1];
      logitw2 = log(sum(exp(theta[3:4]))) - theta[2];
      logitw3 = theta[4] - theta[3];
      
      // if we do not use the likelihood we must restrict the total variance since we use the jeffreys prior
      if (use_likelihood == 0){
        logdens += normal_lpdf(logV | 0, 1);
      } else {
        logdens += jeffrey_logvar_lpdf(logV); // V, total variance
      }
      
      // adding prior for the three logit weights
      logdens += logitw1 - 2*log1p(exp(logitw1)); // w1, residual vs genetic, dirichlet prior with alpha = beta = 1 (which is a uniform[0,1]-prior)
      logdens += eval_spline_lpdf(logitw2 | knots, prior_coeffs, 2, n_knots); // w2, effect 1 vs effect 2 and 3
      logdens += eval_spline_lpdf(logitw3 | knots, prior_coeffs, 3, n_knots); // w3, effect 2 vd effect 3
      
      // jacobian from internal scale to regular scale (log/logit to V/w)
      logdens += logV + log(sum(exp(theta[2:4]))) + log(sum(exp(theta[3:4]))) - sum(theta);
      
      // jacobian from regular scale to variances (from V/w to var)
      logdens += calc_log_jacdet_regtovar(theta);
      
      return(logdens);
      
    }
    
    // from V/w to log variances
    // V = sum of variances
    // w1 = (e^t2 + e^t3 + e^t4)/V
    // w2 = (e^t2)/(e^t2 + e^t3 + e^t4)
    // w3 = (e^t3)/(e^t2 + e^t3 + e^t4)
    real calc_log_jacdet_diri3(real[] theta){
      
      matrix[4,4] jacobian;
      real V;
      
      V = exp(theta[1]) + exp(theta[2]) + exp(theta[3]) + exp(theta[4]);
      
      jacobian[1,1] = exp(theta[1]);
      jacobian[1,2] = exp(theta[2]);
      jacobian[1,3] = exp(theta[3]);
      jacobian[1,4] = exp(theta[4]);
      
      jacobian[2,1] = -exp(theta[1]) * (exp(theta[2]) + exp(theta[3]) + exp(theta[4])) / (V^2);
      jacobian[2,2] = (exp(theta[1]+theta[2])) / (V^2);
      jacobian[2,3] = (exp(theta[1]+theta[3])) / (V^2);
      jacobian[2,4] = (exp(theta[1]+theta[4])) / (V^2);
      
      jacobian[3,1] = 0;
      jacobian[3,2] = exp(theta[2]) * (exp(theta[3]) + exp(theta[4])) / ( (exp(theta[2]) + exp(theta[3]) + exp(theta[4]))^2 );
      jacobian[3,3] = -exp(theta[2])*exp(theta[3]) / ( (exp(theta[2]) + exp(theta[3]) + exp(theta[4]))^2 );
      jacobian[3,4] = -exp(theta[2])*exp(theta[4]) / ( (exp(theta[2]) + exp(theta[3]) + exp(theta[4]))^2 );
      
      jacobian[4,1] = 0;
      jacobian[4,2] = -exp(theta[3])*exp(theta[2]) / ( (exp(theta[2]) + exp(theta[3]) + exp(theta[4]))^2 );
      jacobian[4,3] = exp(theta[3]) * (exp(theta[2]) + exp(theta[4])) / ( (exp(theta[2]) + exp(theta[3]) + exp(theta[4]))^2 );
      jacobian[4,4] = -exp(theta[3])*exp(theta[4]) / ( (exp(theta[2]) + exp(theta[3]) + exp(theta[4]))^2 );
      
      return log(determinant(jacobian));
      
    }
    
    // from V/w to log variances
    // V = sum of variances
    // w1 = (e^t2)/V
    // w2 = (e^t3)/V
    // w3 = (e^t4)/V
    real calc_log_jacdet_diri_ad(real[] theta){
      
      matrix[3,3] jacobian;
      real V;
      
      V = exp(theta[1]) + exp(theta[2]) + exp(theta[3]);
      
      jacobian[1,1] = exp(theta[1]);
      jacobian[1,2] = exp(theta[2]);
      jacobian[1,3] = exp(theta[3]);
      
      jacobian[2,1] = -exp(theta[1]+theta[2])/V^2;
      jacobian[2,2] = exp(theta[2]) * (exp(theta[1])+exp(theta[3])) / (V^2);
      jacobian[2,3] = -exp(theta[3]+theta[2])/V^2;
      
      jacobian[3,1] = -exp(theta[1]+theta[3])/V^2;
      jacobian[3,2] = -exp(theta[2]+theta[3])/V^2;
      jacobian[3,3] = exp(theta[3]) * (exp(theta[1])+exp(theta[2])) / (V^2);
      
      return log(determinant(jacobian));
      
    }
    
    // dirichlet on the triple split weights, new PC on the weight between latent and residuals
    real diri3_lpdf(real[] theta, vector knots, matrix[] prior_coeffs, int n_knots, int use_likelihood){
      
      real logdens;
      real logitw1;
      real logV;
      real logt; // log latent variance
      
      logdens = 0;
      logV = log(sum(exp(theta)));
      logt = log(sum(exp(theta[2:4])));
      logitw1 = log(sum(exp(theta[2:4]))) - theta[1];
      
      // put prior on the weights on log variance scale
      // dirichlet dist is prop. to (w2 w3 (1-w2-w3))^(a-1)
      // w_i = exp(theta_(i+1))/(t), i = 2, 3. t = V-sigma_res^2 = exp(theta[2]) + exp(theta[3]) + exp(theta[4])
      logdens += eval_spline_lpdf(logitw1 | knots, prior_coeffs, 1, n_knots); // w1, residual vs latent
      logdens += (0.754-1) * (sum(theta[2:4]) - 3*logt);
      // add jacobian from V/w to log variances and from logV/logitw1/w2/w3 to V/w
      logdens += calc_log_jacdet_diri3(theta) + (logV -theta[1] -log(exp(theta[2]) + exp(theta[3]) + exp(theta[4])));
      
      // prior on log variance
      if (use_likelihood == 1){
        logdens += jeffrey_logvar_lpdf(logV);
      } else {
        logdens += normal_lpdf(logV | 0, 1);
      }
      
      return(logdens);
      
    }
    
    // dirichlet on all 3 weights, and the jeffreys prior on the total variance
    real diri_lpdf(real[] theta, int use_likelihood){
      
      real logdens;
      real logV;
      
      logdens = 0;
      logV = log(sum(exp(theta)));
      
      // put prior on the weights on log variance scale
      // dirichlet dist is prop. to (w1 w2 (1-w1-w2))^(a-1)
      // w_i = exp(theta_(i+1))/V, i = 1, 2
      logdens += (0.7535-1) * (sum(theta) - 3*logV);
      // add jacobian from V/w to log variances and from logV/w to V/w
      logdens += calc_log_jacdet_diri_ad(theta) -logV;
      
      // prior on log variance
      if (use_likelihood == 1){
        logdens += jeffrey_logvar_lpdf(logV);
      } else {
        logdens += normal_lpdf(logV | 0, 1);
      }
      
      return(logdens);
      
    }
    
}

data {
    
    int<lower=1> n; // number of observations (one observation per individ)
    int<lower=1> n_z; // number of columns in the L_a, L_d, L_e
    
    real y[n]; // data
    
    // Cholesky lower triangle matrix of the covariance matrix for the genetic effect
    matrix[n, n_z] L_a;
    matrix[n, n_z] L_d;
    
    int use_likelihood; // weather to include likelihood or not (not = sample from prior)
    
    // prior stuff
    int n_knots; // number of knots for the splines
    matrix[n_knots, 4] prior_coeffs[2]; // matrix with coefficients for the spline (a + bx + cx^2 + dx^3)
    vector[n_knots] knots; // the knots (logit weight scale) used for the spline (x-values)
    
    // Which prior
    int<lower=1> prior_num;
    // Which order of prior 1
    // int<lower=1, upper=3> order;
    
    // typical variances
    real<lower=0> typvar_a;
    real<lower=0> typvar_d;
    
  }

parameters {
    real intercept; // the intercept
    real theta[3]; // four internal parameters, four log variances (the order is eps, A, D, E)

    // iid effect to be multiplied with the Cholesky lower triangular matrix of V_A, V_D and V_E
    vector[n_z] u_a; // additive
    vector[n_z] u_d; // dominance
    // vector[n_z] u_e; // epistasis
    
    // vector[n] eff_a;
    // vector[n] eff_d;
    // vector[n] eff_e;

    // vector[n_y] eps; // residual noise
}

transformed parameters{
    
    //////// declaring variables
    
    // Linear predictor
    vector[n] eta;
    
    // the three genetic effects, x = L_X * u_x (x = {a, d, e})
    vector[n] eff_a;
    vector[n] eff_d;
    
    // the three genetic effects, x = L_X * u_x (x = {a, d, e})
    vector[n] eff_a_zero;
    vector[n] eff_d_zero;
    
    // Main parameters
    // w1 is between residual (base 1) and genetic 
    // w2 is between A (base 0.5) and D, E
    // w3 is between D (base 0.5) and E
    // (w_c is 1-w, i.e. 1 minus the weight)
    // real<lower=0,upper=1> w1;
    // real<lower=0,upper=1> w2;
    // real<lower=0,upper=1> w1_c; 
    // real<lower=0,upper=1> w2_c;
    
    // real<lower=0> tVar; // total variance
    // real<lower=0> tGen; // total genetic variance

    // Total weights
    // 1, 2 and 3 refers to either ADE, DAE or EAD, depending on the ordering (1, 2 and 3, respectively)
    // real wTot[4]; // to residual (1), genetic 1 (2), genetic 2 (3) and genetic 3 (4)

    // Variances
    real<lower=0> var_eps;
    // variances of the iid vectors u_x 
    // real<lower=0> var1;
    // real<lower=0> var2;
    
    // variances for each effect, which is which internal parameter varies with orderinbg
    real<lower=0> var_A;
    real<lower=0> var_D;
    
    var_eps = exp(theta[1]);
    var_A = exp(theta[2]);
    var_D = exp(theta[3]);
    
    
    // // compute variances
    // var_eps = exp(theta[1]);
    // var1 = exp(theta[2]);
    // var2 = exp(theta[3]);
    // var_eps = exp(theta[1]) * (1-inv_logit(theta[2]));
    // var1 = exp(theta[1]) * inv_logit(theta[2]) * (1-inv_logit(theta[3]));
    // var2 = exp(theta[1]) * inv_logit(theta[2]) * inv_logit(theta[3]) * (1-inv_logit(theta[4]));
    // var3 = exp(theta[1]) * inv_logit(theta[2]) * inv_logit(theta[3]) * inv_logit(theta[4]);
    
    // // Calculate total variances
    // tVar = var_eps + var1 + var2 + var3;
    // tGen = var1 + var2 + var3;
    // 
    // // compute w and 1-w
    // w1 = tGen/tVar;
    // w1_c = var_eps/tVar;
    // w2 = (var2 + var3)/tGen;
    // w2_c = var1/tGen;
    // w3 = var3/(var2 + var3);
    // w3_c = var2/(var2 + var3);
    // 
    // wTot[1] = w1_c; // to residual
    // 
    // if ((prior_num == 1 || prior_num == 2) && order == 2){ // (splitting d alone)
    //   // correct variance to correct parameter
    //   var_A = var2;
    //   var_D = var1;
    //   var_E = var3;
    //   wTot[3] = w1 * w2_c; // to d
    //   wTot[2] = w1 * w2 * w3_c; // to a
    //   wTot[4] = w1 * w2 * w3; // to e
    // } else if ((prior_num == 1 || prior_num == 2) && order == 3){ // (splitting e alone)
    //   // correct variance to correct parameter
    //   var_A = var2;
    //   var_D = var3;
    //   var_E = var1;
    //   wTot[4] = w1 * w2_c; // to e
    //   wTot[2] = w1 * w2 * w3_c; // to a
    //   wTot[3] = w1 * w2 * w3; // to d
    // } else { // prior 3, 4, 5 or 6, or prior 1 or 2 and order 1 (splitting a alone)
    //   // correct variance to correct parameter
    //   var_A = var1;
    //   var_D = var2;
    //   var_E = var3;
    //   wTot[2] = w1 * w2_c; // to a
    //   wTot[3] = w1 * w2 * w3_c; // to d
    //   wTot[4] = w1 * w2 * w3; // to e
    // }
    
    // genetic effects
    eff_a = L_a * u_a;
    eff_d = L_d * u_d;
    
    // Linear predictor 
    // intercept
    eta = rep_vector(intercept, n);
    
    eff_a_zero = eff_a - mean(eff_a);
    eff_d_zero = eff_d - mean(eff_d);
    
    // random effects
    if (var_A > 1e-300) eta += sqrt(var_A) * eff_a_zero;
    if (var_D > 1e-300) eta += sqrt(var_D) * eff_d_zero;

}

model {

    // Prior on intercept
    target += normal_lpdf(intercept | 0, sqrt(1000));
    
    if (prior_num == 1 || prior_num == 2) { // new pc prior, joint prior, either even or uneven base model
        target += new_pc_prior_lpdf(theta | knots, prior_coeffs, n_knots, use_likelihood);
    } else if (prior_num == 3) { // diri3, joint prior NOT USED
        target += diri3_lpdf(theta | knots, prior_coeffs, n_knots, use_likelihood);
    } else if (prior_num == 4) { // diri, joint prior
        target += diri_lpdf(theta | use_likelihood);
    } else if (prior_num == 5) { // old pc
        target += pc_logvar_lpdf(theta[1] | sqrt(1.86*0.75), 0.25);
        target += pc_logvar_lpdf(theta[2] | sqrt(1.86*0.25*0.85), 0.25);
        target += pc_logvar_lpdf(theta[3] | sqrt(1.86*0.25*0.15), 0.25);
    } else if (prior_num == 6) { // old pc ignorant
        target += pc_logvar_lpdf(theta[1] | 0.968, 0.01);
        target += pc_logvar_lpdf(theta[2] | 0.968, 0.01);
        target += pc_logvar_lpdf(theta[3] | 0.968, 0.01);
    } else if (prior_num == 7) { // old pc 0.5 NOT USED
        target += pc_logvar_lpdf(theta[1] | sqrt(1.86*0.5), 0.25);
        target += pc_logvar_lpdf(theta[2] | sqrt(1.86*0.5*0.85), 0.25);
        target += pc_logvar_lpdf(theta[3] | sqrt(1.86*0.5*0.10), 0.25);
    } else if (prior_num == 8) { //NOT USED
      target += new_pc_prior_diri_lpdf(theta | knots, prior_coeffs, n_knots, use_likelihood);
    }
    
    // additive
    target += normal_lpdf(u_a | 0, 1);
    // dominance
    target += normal_lpdf(u_d | 0, 1);
    // 
    // // sum to zero
    // target += normal_lpdf(mean(L_a * u_a) | 0, 1e-4);
    // target += normal_lpdf(mean(L_d * u_d) | 0, 1e-4);
    // target += normal_lpdf(mean(L_e * u_e) | 0, 1e-4);
    // 
    // target += multi_normal_cholesky_lpdf(eff_a | rep_vector(0, n), L_a/typvar_a);
    // target += multi_normal_cholesky_lpdf(eff_d | rep_vector(0, n), L_d/typvar_d);
    // target += multi_normal_cholesky_lpdf(eff_e | rep_vector(0, n), L_e/typvar_a);
    
    // Likelihood
    if (use_likelihood){
        // Observation equation
        target += normal_lpdf(y | eta, sqrt(var_eps));
    }
    
}

generated quantities{
  
    real theta_unscaled[3];
    real tmp_sum[3];
    real var_emp[3];

    // Get random effects
    vector[n] a_effect = sqrt(var_A) * eff_a_zero;
    vector[n] d_effect = sqrt(var_D) * eff_d_zero;
    
    // only ok for order = 1
    theta_unscaled[1] = theta[1];
    theta_unscaled[2] = theta[2];
    theta_unscaled[3] = theta[3];
    
    tmp_sum[1] = 0;
    tmp_sum[2] = 0;
    tmp_sum[3] = 0;
    for (i in 1:n){
      tmp_sum[1] += (a_effect[i] + d_effect[i])^2;
      tmp_sum[2] += a_effect[i]^2;
      tmp_sum[3] += d_effect[i]^2;
    }
    
    var_emp[1] = (tmp_sum[1])/n;
    var_emp[2] = (tmp_sum[2])/n;
    var_emp[3] = (tmp_sum[3])/n;

}


