
// prior 1: HD prior
// prior 2: HD prior dirichlet (ignorant)
// prior 3: old PC pior on all variances, expert
// prior 4: old PC pior on all variances, ignorant

functions {
  
    // calculates the joint PC prior
    real eval_spline_lpdf(real logitw, vector knots, matrix[] prior_coeffs, int idx, int n_knots){
        real dx;
        real val;
        int idd;
        int low;
        int high;
        int med;
        
        // Search for correct cubic polynomial
        low = 1;
        high = n_knots;
        while(high-low > 1){
            med = (high+low)/2;
            if(logitw < knots[med]){
                high = med;
            } else{
                low = med;
            }
        }
        idd = low;
        dx = logitw-knots[idd];
        val =  prior_coeffs[idx, idd, 1];
        val += prior_coeffs[idx, idd, 2]*dx;
        val += prior_coeffs[idx, idd, 3]*dx*dx;
        val += prior_coeffs[idx, idd, 4]*dx*dx*dx;
    
        return(val);
    }
    
    // the old pc prior on log variance
    real pc_logvar_lpdf(real logvar, real u0, real alpha){
        real lambda;
        real val;
        
        // Calculate rate parameter
        lambda = -log(alpha)/u0;
        
        // Calculate prior on standard deviation
        val = gamma_lpdf(exp(0.5*logvar) | 1, lambda);
        
        // Add jacobian
        val += -log(2) + logvar/2;
        
        return(val);
    }
    
    // inverse gamma prior on log variance
    real invgamma_logvar_lpdf(real logvar, real a, real b){
        real val;
        
        // Calculate prior on precision
        val = gamma_lpdf(exp(-logvar) | a, b);
        
        // Add jacobian
        val += -logvar;
        
        return(val);
    }
    
    // gamma distribution on log variance
    real gamma_logvar_lpdf(real x, real a){ // gamma distribution on log variance
        real b = 1;
        return a*log(b) - lgamma(a) + a*x - b*exp(x);
    }
    
    // jeffreys prior
    real jeffrey_logvar_lpdf(real logvar){ // on log variance
      return 0;
    }
    
    // choosing correct prior
    real prior_lpdf(real x, int priorN){
        if(priorN == 2){ // old PC
            return(pc_logvar_lpdf(x | 1, 0.01));
        } else if(priorN == 3){ // gamma
            return(invgamma_logvar_lpdf(x | 1, 5e-5));
        }
        return 0;
    }
    
    // calculates the joint prior for theta (not including covariates)
    real new_pc_prior_lpdf(real[] theta, vector knots, matrix[] prior_coeffs, int n_knots, int use_likelihood){
      
      real logdens;
      real logdens2;
      real logV;
      real logitw;
      
      // initializing
      logdens = 0;
      
      logV = log(sum(exp(theta)));
      // note that logit(x/(x+y)) = log(x) - log(y)
      logitw = theta[2] - theta[1];
      
      // if we do not use the likelihood we must restrict the total variance since we use the jeffreys prior
      if (use_likelihood == 0){
        logdens += normal_lpdf(logV | 0, 1);
      } else {
        logdens += jeffrey_logvar_lpdf(logV); // V, total variance
      }
      
      // calculates the log determinant of the Jacobian of the transformation (log(V), logit(w)) -> (theta_1, theta_2)
      logdens += 0;
      
      // calculates the log determinant of the Jacobian of the transformation (log(V), logit(w)) -> (V, w)
      
      // adding prior for the logit weights
      logdens += eval_spline_lpdf(logitw | knots, prior_coeffs, 1, n_knots); // w1, residual vs genetic/additive

      return(logdens);
      
    }
    
    // dirichlet on weight, jeffreys on total varaince
    // prior is proportional to (exp(t1+t2)/(exp(t1)+exp(t2)))^a (this includes both V and w, and jacobian, i.e. this is the FULL expression)
    real diri_joint_lpdf(real[] x, int use_likelihood){
      
      real logdens; // log density
      real alpha = 1;
      real logV;
      
      logV = log(sum(exp(x)));
      
      // if we do not use the likelihood we must restrict the total variance since we use the jeffreys prior
      if (use_likelihood == 0){
        logdens += normal_lpdf(logV | 0, 1);
      } else {
        logdens += jeffrey_logvar_lpdf(logV); // V, total variance
      }
      
      logdens = alpha*(x[1]+x[2]) - 2*alpha*log(exp(x[1]) + exp(x[2]));
      
      return logdens;
      
    }
    
}

data {
    
    int<lower=1> n; // number of observations (one observation per individ)
    int<lower=1> n_z; // number of columns in the L_a, L_d, L_e
    
    real y[n]; // data
    
    // Cholesky lower triangle matrix of the covariance matrix for the genetic effect
    matrix[n, n_z] L_a;
    
    int use_likelihood; // weather to include likelihood (1) or not (0) (not = sample from prior)
    
    // prior stuff
    int n_knots; // number of knots for the splines
    matrix[n_knots, 4] prior_coeffs[1]; // matrix with coefficients for the spline (a + bx + cx^2 + dx^3)
    vector[n_knots] knots; // the knots (logit weight scale) used for the spline (x-values)
    
    // Which prior
    int<lower=1> prior_num;
    
    // typical variances
    real<lower=0> typvar_a;
    
  }

parameters {
    real intercept; // the intercept
    real theta[2]; // two internal parameters, two log variances (the order is eps, A)

    // iid effect to be multiplied with the Cholesky lower triangular matrix of V_A, V_D and V_E
    vector[n_z] u_a; // additive

    // vector[n_y] eps; // residual noise
}

transformed parameters{
    
    //////// declaring variables
    
    // Linear predictor
    vector[n] eta;
    
    // the three genetic effects, x = L_X * u_x (x = {a, d, e})
    vector[n] eff_a;
    
    // the three genetic effects, x = L_X * u_x (x = {a, d, e})
    vector[n] eff_a_zero;
    
    // Main parameters
    // w1 is between residual (base 1) and genetic 
    // w2 is between A (base 0.5) and D, E
    // w3 is between D (base 0.5) and E
    // (w_c is 1-w, i.e. 1 minus the weight)
    real<lower=0,upper=1> w;
    real<lower=0,upper=1> w_c; 
    
    real<lower=0> tVar; // total variance
    
    // Variances
    real<lower=0> var_eps;
    real<lower=0> var_A;
    
    // compute variances
    var_eps = exp(theta[1]);
    var_A = exp(theta[2]);
    
    // Calculate total variances
    tVar = var_eps + var_A;
    
    // compute w and 1-w
    w = var_A/tVar;
    w_c = var_eps/tVar;
    
    // genetic effects
    eff_a = L_a * u_a;
    
    // Linear predictor 
    // intercept
    eta = rep_vector(intercept, n);
    
    eff_a_zero = eff_a - mean(eff_a);
    
    // random effects
    if (var_A > 1e-300) eta += sqrt(var_A) * eff_a_zero;

}

model {

    // Prior on intercept
    target += normal_lpdf(intercept | 0, sqrt(1000));
    
    if (prior_num == 1){ // HD prior, expert
        target += new_pc_prior_lpdf(theta | knots, prior_coeffs, n_knots, use_likelihood);
    } else if (prior_num == 2) { // HD prior, ignorant
        target += diri_joint_lpdf(theta | use_likelihood);
    } else if (prior_num == 3) { // old pc, expert
        target += pc_logvar_lpdf(theta[1] | sqrt(1.86*0.75), 0.25);
        target += pc_logvar_lpdf(theta[2] | sqrt(1.86*0.25), 0.25);
    } else if (prior_num == 4){ // old pc, ignorant
        target += pc_logvar_lpdf(theta[1] | 0.968, 0.01);
        target += pc_logvar_lpdf(theta[2] | 0.968, 0.01);
    } else if (prior_num == 5) { // old pc, expert 0.5
        target += pc_logvar_lpdf(theta[1] | sqrt(1.86*0.5), 0.25);
        target += pc_logvar_lpdf(theta[2] | sqrt(1.86*0.5), 0.25);
    }
    
    // additive
    target += normal_lpdf(u_a | 0, 1.0);
    
    // Likelihood
    if (use_likelihood){
        // Observation equation
        target += normal_lpdf(y | eta, sqrt(var_eps));
    }
    
}

generated quantities{
  
  
    real theta_unscaled[2];
    real tmp_sum;
    real var_emp;

    // Get random effects
    vector[n] a_effect = sqrt(var_A) * eff_a_zero;
    
    theta_unscaled[1] = theta[1];
    theta_unscaled[2] = theta[2];
    
    tmp_sum = 0;
    for (i in 1:n){
      tmp_sum += a_effect[i]^2;
    }
    
    var_emp = (tmp_sum)/n;

}


